@@ -23,12 +23,18 @@ use ld_so_conf::parse_ldsoconf;
 /// A library dependency
 #[derive(Debug, Clone)]
 pub struct Library {
+    /// Library name
+    pub name: String,
     /// The path to the library.
     pub path: PathBuf,
     /// The normalized real path to the library.
     pub realpath: Option<PathBuf>,
     /// The dependencies of this library.
     pub needed: Vec<String>,
+    /// Runtime library search paths. (deprecated)
+    pub rpath: Vec<String>,
+    /// Runtime library search paths.
+    pub runpath: Vec<String>,
 }
 
 impl Library {
@@ -71,14 +77,12 @@ impl DependencyAnalyzer {
         }
     }
 
-    /// Analyze the given binary.
-    pub fn analyze(mut self, path: impl AsRef<Path>) -> Result<DependencyTree, Error> {
-        let path = path.as_ref();
-        self.load_ld_paths(path)?;
-
-        let bytes = fs::read(path)?;
-        let elf = Elf::parse(&bytes)?;
-
+    fn read_rpath_runpath(
+        &self,
+        elf: &Elf,
+        path: &Path,
+        bytes: &[u8],
+    ) -> Result<(Vec<String>, Vec<String>), Error> {
         let mut rpaths = Vec::new();
         let mut runpaths = Vec::new();
         if let Some(dynamic) = &elf.dynamic {
@@ -100,6 +104,18 @@ impl DependencyAnalyzer {
                 }
             }
         }
+        Ok((rpaths, runpaths))
+    }
+
+    /// Analyze the given binary.
+    pub fn analyze(mut self, path: impl AsRef<Path>) -> Result<DependencyTree, Error> {
+        let path = path.as_ref();
+        self.load_ld_paths(path)?;
+
+        let bytes = fs::read(path)?;
+        let elf = Elf::parse(&bytes)?;
+
+        let (mut rpaths, runpaths) = self.read_rpath_runpath(&elf, path, &bytes)?;
         if !runpaths.is_empty() {
             // If both RPATH and RUNPATH are set, only the latter is used.
             rpaths = Vec::new();
@@ -115,23 +131,29 @@ impl DependencyAnalyzer {
             if libraries.contains_key(&lib_name) {
                 continue;
             }
-            if let Some(library) = self.find_library(&elf, &lib_name)? {
-                libraries.insert(lib_name, library.clone());
-                stack.extend(library.needed);
-            } else {
-                // TODO: return error
-            }
+            let library = self.find_library(&elf, &lib_name)?;
+            libraries.insert(lib_name, library.clone());
+            stack.extend(library.needed);
         }
 
         let interpreter = elf.interpreter.map(|interp| interp.to_string());
         if let Some(ref interp) = interpreter {
             if !libraries.contains_key(interp) {
+                let interp_path = PathBuf::from(interp);
+                let interp_name = interp_path
+                    .file_name()
+                    .expect("missing filename")
+                    .to_str()
+                    .expect("Filename isn't valid Unicode");
                 libraries.insert(
                     interp.to_string(),
                     Library {
-                        path: PathBuf::from(interp),
+                        name: interp_name.to_string(),
+                        path: interp_path,
                         realpath: PathBuf::from(interp).canonicalize().ok(),
                         needed: Vec::new(),
+                        rpath: Vec::new(),
+                        runpath: Vec::new(),
                     },
                 );
             }
@@ -190,8 +212,8 @@ impl DependencyAnalyzer {
         Ok(())
     }
 
-    /// Try to locate a `lib` taht is compatible to `elf`
-    fn find_library(&self, elf: &Elf, lib: &str) -> Result<Option<Library>, Error> {
+    /// Try to locate a `lib` that is compatible to `elf`
+    fn find_library(&self, elf: &Elf, lib: &str) -> Result<Library, Error> {
         for ld_path in self
             .runpaths
             .iter()
@@ -205,19 +227,26 @@ impl DependencyAnalyzer {
                 let lib_elf = Elf::parse(&bytes)?;
                 if compatible_elfs(elf, &lib_elf) {
                     let needed = lib_elf.libraries.iter().map(ToString::to_string).collect();
-                    return Ok(Some(Library {
+                    let (rpath, runpath) = self.read_rpath_runpath(&lib_elf, &lib_path, &bytes)?;
+                    return Ok(Library {
+                        name: lib.to_string(),
                         path: lib_path.to_path_buf(),
                         realpath: lib_path.canonicalize().ok(),
                         needed,
-                    }));
+                        rpath,
+                        runpath,
+                    });
                 }
             }
         }
-        Ok(Some(Library {
+        Ok(Library {
+            name: lib.to_string(),
             path: PathBuf::from(lib),
             realpath: None,
             needed: Vec::new(),
-        }))
+            rpath: Vec::new(),
+            runpath: Vec::new(),
+        })
     }
 }
 
